@@ -1,7 +1,7 @@
 ### -*- coding: utf-8 -*- ####################################################
 ##############################################################################
 #
-# Copyright (c) 2008-2009 Thierry Florac <tflorac AT ulthar.net>
+# Copyright (c) 2008-2011 Thierry Florac <tflorac AT ulthar.net>
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -16,15 +16,16 @@
 __docformat__ = "restructuredtext"
 
 # import standard packages
+import transaction
 
 # import Zope3 interfaces
-from zope.app.appsetup.interfaces import IDatabaseOpenedWithRootEvent
-from zope.app.catalog.interfaces import ICatalog
-from zope.app.intid.interfaces import IIntIds
 from zope.app.publication.zopepublication import ZopePublication
+from zope.catalog.interfaces import ICatalog
 from zope.component.interfaces import IComponentRegistry
 from zope.dublincore.interfaces import IZopeDublinCore
+from zope.intid.interfaces import IIntIds
 from zope.location.interfaces import ISite
+from zope.processlifetime import IDatabaseOpenedWithRoot
 
 # import local interfaces
 from hurry.workflow.interfaces import IWorkflowState
@@ -32,18 +33,17 @@ from ztfy.utils.interfaces import INewSiteManagerEvent
 
 # import Zope3 packages
 from zc.catalog.catalogindex import ValueIndex, DateTimeValueIndex
-from zope.app.catalog.catalog import Catalog
-from zope.app.component import hooks
-from zope.app.generations.utility import findObjectsProviding
-from zope.app.intid import IntIds
-from zope.component import adapter
+from zope.catalog.catalog import Catalog
+from zope.component import adapter, queryUtility
+from zope.intid import IntIds
 from zope.location import locate
+from zope.site import hooks
 
 # import local packages
 from ztfy.utils.site import locateAndRegister
 
 
-def updateDatabaseIfNeeded(context, create_catalog=False):
+def updateDatabaseIfNeeded(context):
     """Check for missing objects at application startup"""
     try:
         sm = context.getSiteManager()
@@ -51,15 +51,15 @@ def updateDatabaseIfNeeded(context, create_catalog=False):
         return
     default = sm['default']
     # Check for required IIntIds utility
-    intids = default.get('IntIds')
+    intids = queryUtility(IIntIds)
     if intids is None:
         intids = IntIds()
         locate(intids, default)
-        IComponentRegistry(sm).registerUtility(intids, IIntIds, '')
+        IComponentRegistry(sm).registerUtility(intids, IIntIds)
         default['IntIds'] = intids
     # Check for required catalog and index
     catalog = default.get('WorkflowCatalog')
-    if (catalog is None) and create_catalog:
+    if catalog is None:
         catalog = Catalog()
         locateAndRegister(catalog, default, 'WorkflowCatalog', intids)
         IComponentRegistry(sm).registerUtility(catalog, ICatalog, 'WorkflowCatalog')
@@ -84,23 +84,19 @@ def updateDatabaseIfNeeded(context, create_catalog=False):
             locateAndRegister(index, catalog, 'expiration_date', intids)
 
 
-@adapter(IDatabaseOpenedWithRootEvent)
+@adapter(IDatabaseOpenedWithRoot)
 def handleOpenedDatabase(event):
     db = event.database
     connection = db.open()
     root = connection.root()
     root_folder = root.get(ZopePublication.root_name, None)
-    updated = False
-    for site in findObjectsProviding(root_folder, ISite):
-        if site is not root_folder:
+    for site in root_folder.values():
+        if ISite(site, None) is not None:
             hooks.setSite(site)
             updateDatabaseIfNeeded(site)
-            updated = True
-    if not updated:
-        hooks.setSite(root_folder)
-        updateDatabaseIfNeeded(root_folder)
+            transaction.commit()
 
 
 @adapter(INewSiteManagerEvent)
 def handleNewSiteManager(event):
-    updateDatabaseIfNeeded(event.object, create_catalog=True)
+    updateDatabaseIfNeeded(event.object)
